@@ -1,7 +1,7 @@
  # ====================================================================
  # The Apache Software License, Version 1.1
  #
- # Copyright (c) 2000-2002 The Apache Software Foundation.  All rights
+ # Copyright (c) 2000-2003 The Apache Software Foundation.  All rights
  # reserved.
  #
  # Redistribution and use in source and binary forms, with or without
@@ -54,7 +54,7 @@
  #
  # Originally developed by Gregory Trubetskoy.
  #
- # $Id: apache.py,v 1.68 2003/02/28 04:55:15 grisha Exp $
+ # $Id: apache.py,v 1.75 2003/08/26 01:56:39 grisha Exp $
 
 import sys
 import traceback
@@ -98,12 +98,12 @@ class CallBack:
 
         # config
         config = conn.base_server.get_config()
-        debug = config.has_key("PythonDebug")
+        debug = int(config.get("PythonDebug", 0))
 
         try:
 
             handler = conn.hlist.handler
-            
+
             # split module::handler
             l = handler.split('::', 1)
             module_name = l[0]
@@ -115,7 +115,7 @@ class CallBack:
 
             # add the directory to pythonpath if
             # not there yet, or pythonpath specified
-            
+
             if config.has_key("PythonPath"):
                 # we want to do as little evaling as possible,
                 # so we remember the path in un-evaled form and
@@ -129,8 +129,9 @@ class CallBack:
                         sys.path[:] = newpath
 
             # import module
-            module = import_module(module_name, config)
-
+            module = import_module(module_name,
+                                   autoreload=int(config.get("PythonAutoReload", 1)),
+                                   log=debug)
             # find the object
             object = resolve_object(module, object_str,
                                     arg=conn, silent=0)
@@ -161,7 +162,8 @@ class CallBack:
             try:
                 exc_type, exc_value, exc_traceback = sys.exc_info()
                 result = self.ReportError(exc_type, exc_value, exc_traceback, srv=conn.base_server,
-                                          phase=filter.name, hname=handler, debug=debug)
+                                          phase="ConnectionHandler",
+                                          hname=handler, debug=debug)
             finally:
                 exc_traceback = None
 
@@ -173,7 +175,7 @@ class CallBack:
 
         # config
         config = req.get_config()
-        debug = config.has_key("PythonDebug")
+        debug = int(config.get("PythonDebug", 0))
 
         try:
 
@@ -208,7 +210,9 @@ class CallBack:
                     sys.path[:0] = [filter.dir]
 
             # import module
-            module = import_module(module_name, config)
+            module = import_module(module_name,
+                                   autoreload=int(config.get("PythonAutoReload", 1)),
+                                   log=debug)
 
             # find the object
             object = resolve_object(module, object_str, arg=filter, silent=0)
@@ -281,7 +285,7 @@ class CallBack:
 
         # config
         config = req.get_config()
-        debug = config.has_key("PythonDebug")
+        debug = int(config.get("PythonDebug", 0))
 
         try:
             hlist = req.hlist
@@ -317,7 +321,9 @@ class CallBack:
                         sys.path[:0] = [dir]
 
                 # import module
-                module = import_module(module_name, config)
+                module = import_module(module_name,
+                                       autoreload=int(config.get("PythonAutoReload", 1)),
+                                       log=debug)
 
                 # find the object
                 object = resolve_object(module, object_str,
@@ -415,11 +421,12 @@ class CallBack:
                     return HTTP_INTERNAL_SERVER_ERROR
                 else:
                     # write to client
-                    req.content_type = 'text/plain'
+                    req.content_type = 'text/html'
 
-                    s = '\nMod_python error: "%s %s"\n\n' % (phase, hname)
+                    s = '\n<pre>\nMod_python error: "%s %s"\n\n' % (phase, hname)
                     for e in traceback.format_exception(etype, evalue, etb):
                         s = s + e + '\n'
+                    s = s + "</pre>\n"
                         
                     if filter:
                         filter.write(s)
@@ -437,20 +444,13 @@ class CallBack:
             etb = None
             # we do not return anything
 
-def import_module(module_name, config=None, path=None):
+def import_module(module_name, autoreload=1, log=0, path=None):
     """
     Get the module to handle the request. If
     autoreload is on, then the module will be reloaded
     if it has changed since the last import.
     """
 
-    # Get options
-    debug, autoreload = 0, 1
-    if config:
-        debug = config.has_key("PythonDebug")
-        if config.has_key("PythonAutoReload"):
-            autoreload = int(config["PythonAutoReload"])
-
     # (Re)import
     if sys.modules.has_key(module_name):
 
@@ -484,7 +484,7 @@ def import_module(module_name, config=None, path=None):
     if mtime > oldmtime:
 
         # Import the module
-        if debug:
+        if log:
             if path:
                 s = "mod_python: (Re)importing module '%s' with path set to '%s'" % (module_name, path)
             else:
@@ -763,7 +763,7 @@ def restore_nocgi(sav_env, si, so):
     for k in osenv.keys():
         del osenv[k]
     for k in sav_env:
-        osenv[k] = env[k]
+        osenv[k] = sav_env[k]
 
     sys.stdout = si
     sys.stdin = so
@@ -781,6 +781,7 @@ log_error = _apache.log_error
 table = _apache.table
 config_tree = _apache.config_tree
 server_root = _apache.server_root
+mpm_query = _apache.mpm_query
 
 ## Some constants
 
@@ -940,20 +941,27 @@ AP_REQ_REJECT_PATH_INFO = 1  # Send 404 error if path_info was given
 AP_REQ_DEFAULT_PATH_INFO = 2 # Module's choice for handling path_info
 
 
+# for mpm_query
+AP_MPMQ_NOT_SUPPORTED      = 0  # This value specifies whether 
+                                # an MPM is capable of         
+                                # threading or forking.        
+AP_MPMQ_STATIC             = 1  # This value specifies whether 
+                                # an MPM is using a static # of
+                                # threads or daemons.          
+AP_MPMQ_DYNAMIC            = 2  # This value specifies whether 
+                                # an MPM is using a dynamic # of
+                                # threads or daemons.          
 
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
+AP_MPMQ_MAX_DAEMON_USED    = 1  # Max # of daemons used so far 
+AP_MPMQ_IS_THREADED        = 2  # MPM can do threading         
+AP_MPMQ_IS_FORKED          = 3  # MPM can do forking           
+AP_MPMQ_HARD_LIMIT_DAEMONS = 4  # The compiled max # daemons   
+AP_MPMQ_HARD_LIMIT_THREADS = 5  # The compiled max # threads   
+AP_MPMQ_MAX_THREADS        = 6  # # of threads/child by config 
+AP_MPMQ_MIN_SPARE_DAEMONS  = 7  # Min # of spare daemons       
+AP_MPMQ_MIN_SPARE_THREADS  = 8  # Min # of spare threads       
+AP_MPMQ_MAX_SPARE_DAEMONS  = 9  # Max # of spare daemons       
+AP_MPMQ_MAX_SPARE_THREADS  = 10 # Max # of spare threads       
+AP_MPMQ_MAX_REQUESTS_DAEMON= 11 # Max # of requests per daemon 
+AP_MPMQ_MAX_DAEMONS        = 12 # Max # of daemons by config   
+                                                                                                                                                                                                                                    
