@@ -57,7 +57,7 @@
  *
  * mod_python.c 
  *
- * $Id: mod_python.c,v 1.87 2003/01/09 19:43:56 grisha Exp $
+ * $Id: mod_python.c,v 1.99 2003/08/21 18:25:12 grisha Exp $
  *
  * See accompanying documentation and source code comments 
  * for details.
@@ -320,6 +320,132 @@ apr_status_t python_cleanup(void *data)
     return APR_SUCCESS;
 }
 
+static apr_status_t init_mutexes(server_rec *s, apr_pool_t *p, py_global_config *glb)
+{
+    int max_threads;
+    int max_procs;
+    int max_clients;
+    int locks;
+    int n;
+
+    /* figre out maximum possible concurrent connections */
+    ap_mpm_query(AP_MPMQ_MAX_THREADS, &max_threads);
+    ap_mpm_query(AP_MPMQ_MAX_DAEMONS, &max_procs);
+    max_clients = (((!max_threads) ? 1 : max_threads) *
+		   ((!max_procs) ? 1 : max_procs));
+    locks = max_clients; /* XXX this is completely out of the blue */
+
+    ap_log_error(APLOG_MARK, APLOG_NOTICE, 0, s,
+		 "mod_python: Creating %d session mutexes based "
+		 "on %d max processes and %d max threads.", 
+		 locks, max_procs, max_threads);
+
+    glb->g_locks = (apr_global_mutex_t **) 
+	apr_palloc(p, locks * sizeof(apr_global_mutex_t *));
+    glb->nlocks = locks;
+    glb->parent_pid = getpid();
+
+    for (n=0; n<locks; n++) {
+	apr_status_t rc;
+	apr_global_mutex_t **mutex = glb->g_locks;
+
+#if !defined(OS2) && !defined(WIN32) && !defined(BEOS) && !defined(NETWARE)
+	char fname[255];
+
+	snprintf(fname, 255, "/tmp/mpmtx%d%d", glb->parent_pid, n);
+#else
+	char *fname = NULL;
+#endif
+	rc = apr_global_mutex_create(&mutex[n], fname, APR_LOCK_DEFAULT, 
+				     p);
+	if (rc != APR_SUCCESS) {
+	    ap_log_error(APLOG_MARK, APLOG_ERR, rc, s,
+			 "mod_python: Failed to create global mutex %d of %d (%s).",
+			 n, locks, (!fname) ? "<null>" : fname);
+	    if (n > 0) {
+		/* we were able to crate at least one, so lets just print a 
+		   warning/hint and proceed
+		*/
+		ap_log_error(APLOG_MARK, APLOG_ERR, 0, s,
+			     "mod_python: We can probably continue, but with diminished ability "
+			     "to process session locks.");
+		ap_log_error(APLOG_MARK, APLOG_ERR, 0, s,
+			     "mod_python: Hint: On Linux, the problem may be the number of "
+			     "available semaphores, check 'sysctl kernel.sem'");
+		glb->nlocks = n;
+		break;
+		
+	    }
+	    else {
+		return rc;
+	    }
+	}
+	else {
+#if !defined(OS2) && !defined(WIN32) && !defined(BEOS) && !defined(NETWARE)
+	    chown(fname, unixd_config.user_id, -1);
+#endif
+	}
+    }
+    return APR_SUCCESS;
+}
+
+static apr_status_t reinit_mutexes(server_rec *s, apr_pool_t *p, py_global_config *glb)
+{
+    int n;
+
+    for (n=0; n< glb->nlocks; n++) {
+	apr_status_t rc;
+	apr_global_mutex_t **mutex = glb->g_locks;
+
+#if !defined(OS2) && !defined(WIN32) && !defined(BEOS) && !defined(NETWARE)
+        char fname[255];
+        snprintf(fname, 255, "/tmp/mpmtx%d%d", glb->parent_pid, n);
+#else
+        char *fname = NULL;
+#endif
+	rc = apr_global_mutex_child_init(&mutex[n], fname, p);
+
+	if (rc != APR_SUCCESS) {
+	    ap_log_error(APLOG_MARK, APLOG_STARTUP, rc, s,
+			 "mod_python: Failed to reinit global mutex %s.",
+			 (!fname) ? "<null>" : fname);
+	    return rc;
+	}
+    }
+    return APR_SUCCESS;
+}
+
+/**
+ ** python_create_global_config
+ **
+ *      This creates the part of config that survives
+ *  server restarts
+ *
+ */
+
+static py_global_config *python_create_global_config(server_rec *s)
+{
+    apr_pool_t *pool = s->process->pool;
+    py_global_config *glb;
+
+    /* do we already have it in s->process->pool? */
+    apr_pool_userdata_get((void **)&glb, MP_CONFIG_KEY, pool);
+
+    if (glb) {
+        return glb; 
+    }
+
+    /* otherwise, create it */
+
+    glb = (py_global_config *)apr_palloc(pool, sizeof(*glb));
+
+    apr_pool_userdata_set(glb, MP_CONFIG_KEY,
+                          apr_pool_cleanup_null,
+                          pool);
+
+    return glb;
+}
+
 /**
  ** python_init()
  **
@@ -329,10 +455,11 @@ apr_status_t python_cleanup(void *data)
 static int python_init(apr_pool_t *p, apr_pool_t *ptemp, 
                        apr_pool_t *plog, server_rec *s)
 {
-
     char buff[255];
     void *data;
+    py_global_config *glb;
     const char *userdata_key = "python_init";
+    apr_status_t rc;
 
     apr_pool_userdata_get(&data, userdata_key, s->process->pool);
     if (!data) {
@@ -348,6 +475,12 @@ static int python_init(apr_pool_t *p, apr_pool_t *ptemp,
     sprintf(buff, "Python/%.200s", strtok((char *)Py_GetVersion(), " "));
     ap_add_version_component(p, buff);
 
+    /* global config */
+    glb = python_create_global_config(s);
+    if ((rc = init_mutexes(s, p, glb)) != APR_SUCCESS) {
+	return rc;
+    }
+
     /* initialize global Python interpreter if necessary */
     if (! Py_IsInitialized()) 
     {
@@ -375,6 +508,7 @@ static int python_init(apr_pool_t *p, apr_pool_t *ptemp,
         /* release the lock; now other threads can run */
         PyEval_ReleaseLock();
 #endif
+
     }
     return OK;
 }
@@ -663,37 +797,37 @@ static requestobject *get_request_object(request_rec *req, const char *interp_na
         request_obj = req_config->request_obj;
     }
     else {
-        if ((req->path_info) && 
-            (req->path_info[strlen(req->path_info) - 1] == SLASH))
-        {
-            int i;
-            i = strlen(req->path_info);
-            /* take out the slash */
-            req->path_info[i - 1] = 0;
+/*         if ((req->path_info) &&  */
+/*             (req->path_info[strlen(req->path_info) - 1] == SLASH)) */
+/*         { */
+/*             int i; */
+/*             i = strlen(req->path_info); */
+/*             /\* take out the slash *\/ */
+/*             req->path_info[i - 1] = 0; */
 
+/*             Py_BEGIN_ALLOW_THREADS */
+/*             ap_add_cgi_vars(req); */
+/*             Py_END_ALLOW_THREADS */
+
+/*             request_obj = (requestobject *)MpRequest_FromRequest(req); */
+/*             if (!request_obj) return NULL; */
+
+/*             /\* put the slash back in *\/ */
+/*             req->path_info[i - 1] = SLASH;  */
+/*             req->path_info[i] = 0; */
+
+/*             /\* and also make PATH_INFO == req->subprocess_env *\/ */
+/*             apr_table_set(req->subprocess_env, "PATH_INFO", req->path_info); */
+/*         }  */
+/*         else  */
+/*         {  */
             Py_BEGIN_ALLOW_THREADS
             ap_add_cgi_vars(req);
             Py_END_ALLOW_THREADS
 
             request_obj = (requestobject *)MpRequest_FromRequest(req);
             if (!request_obj) return NULL;
-
-            /* put the slash back in */
-            req->path_info[i - 1] = SLASH; 
-            req->path_info[i] = 0;
-
-            /* and also make PATH_INFO == req->subprocess_env */
-            apr_table_set(req->subprocess_env, "PATH_INFO", req->path_info);
-        } 
-        else 
-        { 
-            Py_BEGIN_ALLOW_THREADS
-            ap_add_cgi_vars(req);
-            Py_END_ALLOW_THREADS
-
-            request_obj = (requestobject *)MpRequest_FromRequest(req);
-            if (!request_obj) return NULL;
-        }
+/*         } */
 
         /* store the pointer to this object in request_config */
         req_config = apr_pcalloc(req->pool, sizeof(py_req_config));
@@ -1578,9 +1712,9 @@ static void PythonChildInitHandler(apr_pool_t *p, server_rec *s)
 
 
     hl_entry *hle;
-
     py_config *conf = ap_get_module_config(s->module_config, &python_module);
-    
+    py_global_config *glb;
+
     /* accordig Py C Docs we must do this after forking */
 
 #ifdef WITH_THREAD
@@ -1593,11 +1727,20 @@ static void PythonChildInitHandler(apr_pool_t *p, server_rec *s)
 
     /*
      * Cleanups registered first will be called last. This will
-     * end the Python enterpreter *after* all other cleanups.
+     * end the Python interpreter *after* all other cleanups.
      */
 
     apr_pool_cleanup_register(p, NULL, python_finalize, apr_pool_cleanup_null);
 
+    /*
+     * Reinit mutexes
+     */
+
+    /* this will return it if it already exists */
+    glb = python_create_global_config(s);
+
+    reinit_mutexes(s, p, glb);
+
     /*
      * remember the pool in a global var. we may use it
      * later in server.register_cleanup()
@@ -1693,7 +1836,8 @@ static int PythonHandler(request_rec *req) {
      * handle those that we explicitly agreed to handle (see 
      * above).
      */
-    if (!req->handler || strcmp(req->handler, "python-program"))
+    if (!req->handler || (strcmp(req->handler, "mod_python") &&
+			  strcmp(req->handler, "python-program")))
         return DECLINED;
 
     return python_handler(req, "PythonHandler");
