@@ -57,7 +57,7 @@
  *
  * requestobject.c 
  *
- * $Id: requestobject.c,v 1.44 2003/03/07 20:04:30 grisha Exp $
+ * $Id: requestobject.c,v 1.50 2003/08/04 14:54:46 grisha Exp $
  *
  */
 
@@ -422,7 +422,7 @@ static PyObject * req_log_error(requestobject *self, PyObject *args)
     int level = 0;
     char *message = NULL;
 
-    if (! PyArg_ParseTuple(args, "z|iO", &message, &level))
+    if (! PyArg_ParseTuple(args, "z|i", &message, &level))
         return NULL; /* error */
 
     if (message) {
@@ -496,6 +496,12 @@ static PyObject * req_read(requestobject *self, PyObject *args)
     while ((self->rbuff_pos < self->rbuff_len) && (copied < len))
         buffer[copied++] = self->rbuff[self->rbuff_pos++];
     
+    /* Free rbuff if we're done with it */
+    if (self->rbuff_pos >= self->rbuff_len && self->rbuff != NULL) {
+	free(self->rbuff);
+	self->rbuff = NULL;
+    }
+
     if (copied == len)
         return result;  /* we're done! */
 
@@ -607,12 +613,21 @@ static PyObject * req_readline(requestobject *self, PyObject *args)
         }
     }
 
+    /* Free old rbuff as the old contents have been copied over and
+       we are about to allocate a new rbuff. Perhaps this could be reused
+       somehow? */
+    if (self->rbuff_pos >= self->rbuff_len && self->rbuff != NULL)
+    {
+	free(self->rbuff);
+	self->rbuff = NULL;
+    }
+
     /* if got this far, the buffer should be empty, we need to read more */
         
     /* create a read buffer */
     self->rbuff_len = len > HUGE_STRING_LEN ? len : HUGE_STRING_LEN;
     self->rbuff_pos = self->rbuff_len;
-    self->rbuff = apr_palloc(self->request_rec->pool, self->rbuff_len);
+    self->rbuff = malloc(self->rbuff_len);
     if (! self->rbuff)
         return PyErr_NoMemory();
 
@@ -633,6 +648,11 @@ static PyObject * req_readline(requestobject *self, PyObject *args)
         Py_END_ALLOW_THREADS
 
         if (chunk_len == -1) {
+
+	    /* Free rbuff since returning NULL here should end the request */
+	    free(self->rbuff);
+	    self->rbuff = NULL;
+
             PyErr_SetObject(PyExc_IOError, 
                             PyString_FromString("Client read error (Timeout?)"));
             return NULL;
@@ -653,6 +673,13 @@ static PyObject * req_readline(requestobject *self, PyObject *args)
             break;
     }
 
+    /* Free rbuff if we're done with it */
+    if (self->rbuff_pos >= self->rbuff_len && self->rbuff != NULL)
+    {
+	free(self->rbuff);
+	self->rbuff = NULL;
+    }
+
     /* resize if necessary */
     if (copied < len) 
         if(_PyString_Resize(&result, copied))
@@ -793,13 +820,16 @@ static PyObject * req_write(requestobject *self, PyObject *args)
     if (! PyArg_ParseTuple(args, "s#", &buff, &len))
         return NULL;  /* bad args */
 
-    Py_BEGIN_ALLOW_THREADS
-    ap_rwrite(buff, len, self->request_rec);
-    rc = ap_rflush(self->request_rec);
-    Py_END_ALLOW_THREADS
-    if (rc == EOF) {
-        PyErr_SetString(PyExc_IOError, "Write failed, client closed connection.");
-        return NULL;
+    if (len > 0 ) {
+
+        Py_BEGIN_ALLOW_THREADS
+        ap_rwrite(buff, len, self->request_rec);
+        rc = ap_rflush(self->request_rec);
+        Py_END_ALLOW_THREADS
+            if (rc == EOF) {
+                PyErr_SetString(PyExc_IOError, "Write failed, client closed connection.");
+                return NULL;
+            }
     }
 
     Py_INCREF(Py_None);
@@ -807,6 +837,54 @@ static PyObject * req_write(requestobject *self, PyObject *args)
 
 }
 
+static PyObject * req_sendfile(requestobject *self, PyObject *args)
+{
+    char *fname;
+    apr_file_t *fd;
+    apr_size_t offset=0, len=-1, nbytes;
+    apr_status_t status;
+    PyObject * py_result = NULL;
+    apr_finfo_t finfo;
+    
+    if (! PyArg_ParseTuple(args, "s|ll", &fname, &offset, &len))
+        return NULL;  /* bad args */
+
+    Py_BEGIN_ALLOW_THREADS
+    status=apr_stat(&finfo, fname,
+                    APR_READ, self->request_rec->pool);
+    Py_END_ALLOW_THREADS
+    if (status != APR_SUCCESS) {
+        PyErr_SetString(PyExc_IOError, "Could not stat file for reading");
+        return NULL;
+    }
+    
+    Py_BEGIN_ALLOW_THREADS                         
+    status=apr_file_open(&fd, fname,
+                         APR_READ, finfo.protection,
+                         self->request_rec->pool);
+    Py_END_ALLOW_THREADS
+    if (status != APR_SUCCESS) {
+        PyErr_SetString(PyExc_IOError, "Could not open file for reading");
+        return NULL;
+    }                         
+    
+    if (len==-1) len=finfo.size;
+        
+    Py_BEGIN_ALLOW_THREADS                         
+        status = ap_send_fd(fd, self->request_rec, offset, 
+                            len, &nbytes);
+    Py_END_ALLOW_THREADS
+    
+    if (status != APR_SUCCESS) {
+        PyErr_SetString(PyExc_IOError, "Write failed, client closed connection.");
+        return NULL;
+    }
+
+    py_result = PyLong_FromLong (nbytes);
+    Py_INCREF(py_result);
+    return py_result;
+}
+
 static PyMethodDef request_methods[] = {
     {"add_common_vars",       (PyCFunction) req_add_common_vars,       METH_NOARGS},
     {"add_handler",           (PyCFunction) req_add_handler,           METH_VARARGS},
@@ -824,55 +902,12 @@ static PyMethodDef request_methods[] = {
     {"readlines",             (PyCFunction) req_readlines,             METH_VARARGS},
     {"register_cleanup",      (PyCFunction) req_register_cleanup,      METH_VARARGS},
     {"send_http_header",      (PyCFunction) req_send_http_header,      METH_NOARGS},
+    {"sendfile",              (PyCFunction) req_sendfile,              METH_VARARGS},
     {"set_content_length",    (PyCFunction) req_set_content_length,    METH_VARARGS},
     {"write",                 (PyCFunction) req_write,                 METH_VARARGS},
     { NULL, NULL } /* sentinel */
 };
 
-/**
- ** getmakeobj
- **
- *    A getter func that creates an object as needed.
- */
-
-static PyObject *getmakeobj(requestobject* self, void *objname) 
-{
-    char *name = (char *)objname;
-    PyObject *result = NULL;
-
-    if (strcmp(name, "connection") == 0) {
-        if (!self->connection && self->request_rec->connection)
-            self->connection = MpConn_FromConn(self->request_rec->connection);
-        result = self->connection;
-    }
-    else if (strcmp(name, "server") == 0) {
-        if (!self->server && self->request_rec->server) 
-            self->server = MpServer_FromServer(self->request_rec->server);
-        result = self->server;
-    }
-    else if (strcmp(name, "next") == 0) {
-        if (!self->next && self->request_rec->next)
-            self->next = MpRequest_FromRequest(self->request_rec->next);
-        result = self->next;
-    }
-    else if (strcmp(name, "prev") == 0) {
-        if (!self->prev && self->request_rec->prev)
-            self->prev = MpRequest_FromRequest(self->request_rec->prev);
-        result = self->prev;
-    }
-    else if (strcmp(name, "main") == 0) {
-        if (!self->main && self->request_rec->main)
-            self->main = MpRequest_FromRequest(self->request_rec->main);
-        result = self->main;
-    }
-
-    if (!result)
-        result = Py_None;
-
-    Py_INCREF(result);
-    return result;
-
-}
 
 /* 
    These are offsets into the Apache request_rec structure.
@@ -1122,6 +1157,51 @@ static PyObject *getreq_rec_uri(requestobject *self, void *name)
     return tuple_from_apr_uri(uri);
 }
 
+/**
+ ** getmakeobj
+ **
+ *    A getter func that creates an object as needed.
+ */
+
+static PyObject *getmakeobj(requestobject* self, void *objname) 
+{
+    char *name = (char *)objname;
+    PyObject *result = NULL;
+
+    if (strcmp(name, "connection") == 0) {
+        if (!self->connection && self->request_rec->connection)
+            self->connection = MpConn_FromConn(self->request_rec->connection);
+        result = self->connection;
+    }
+    else if (strcmp(name, "server") == 0) {
+        if (!self->server && self->request_rec->server) 
+            self->server = MpServer_FromServer(self->request_rec->server);
+        result = self->server;
+    }
+    else if (strcmp(name, "next") == 0) {
+        if (!self->next && self->request_rec->next)
+            self->next = MpRequest_FromRequest(self->request_rec->next);
+        result = self->next;
+    }
+    else if (strcmp(name, "prev") == 0) {
+        if (!self->prev && self->request_rec->prev)
+            self->prev = MpRequest_FromRequest(self->request_rec->prev);
+        result = self->prev;
+    }
+    else if (strcmp(name, "main") == 0) {
+        if (!self->main && self->request_rec->main)
+            self->main = MpRequest_FromRequest(self->request_rec->main);
+        result = self->main;
+    }
+
+    if (!result)
+        result = Py_None;
+
+    Py_INCREF(result);
+    return result;
+
+}
+
 static PyGetSetDef request_getsets[] = {
     {"connection", (getter)getmakeobj, NULL, "Connection object", "connection"},
     {"server",     (getter)getmakeobj, NULL, "Server object", "server"},
